@@ -21,15 +21,15 @@
  * at the appropriate times to change the pointers to these servers.
  */
 typedef struct cm_server {
-	struct cm_server *allNextp;		/* locked by cm_serverLock */
+    struct cm_server *allNextp;		/* locked by cm_serverLock */
     struct sockaddr_in addr;		/* by mx */
     int type;				/* by mx */
-	struct cm_conn *connsp;			/* locked by cm_connLock */
+    struct cm_conn *connsp;		/* locked by cm_connLock */
     long flags;				/* by mx */
-    struct cm_cell *cellp;			/* cell containing this server */
-	int refCount;				/* locked by cm_serverLock */
-    osi_mutex_t mx;
-	unsigned short ipRank;			/* server priority */
+    struct cm_cell *cellp;		/* cell containing this server */
+    unsigned long refCount;		/* locked by cm_serverLock */
+    osi_mutex_t mx;                     
+    unsigned short ipRank;		/* server priority */
 } cm_server_t;
 
 enum repstate {not_busy, busy, offline};
@@ -38,7 +38,7 @@ typedef struct cm_serverRef {
 	struct cm_serverRef *next;      /* locked by cm_serverLock */
 	struct cm_server *server;       /* locked by cm_serverLock */
 	enum repstate status;           /* locked by cm_serverLock */
-    int refCount;                   /* locked by cm_serverLock */
+    unsigned long refCount;                   /* locked by cm_serverLock */
 } cm_serverRef_t;
 
 /* types */
@@ -74,6 +74,10 @@ extern cm_serverRef_t *cm_NewServerRef(struct cm_server *serverp);
 
 extern long cm_ChecksumServerList(cm_serverRef_t *serversp);
 
+extern void cm_GetServer(cm_server_t *);
+
+extern void cm_GetServerNoLock(cm_server_t *);
+
 extern void cm_PutServer(cm_server_t *);
 
 extern void cm_PutServerNoLock(cm_server_t *);
